; The GIMP -- an image manipulation program
; Copyright (C) 1995 Spencer Kimball and Peter Mattis
; 
; Grey point script
; Version 1.0
; Date 29/09/2004
; Copyright (C) 2004 Dr. Martin Rogge <marogge@onlinehome.de>
; This is my first Scheme program, so take it easy. ;^)
;
; Version 1.0a Raymond Ostertag 2004/10
;   - changed menu entry
 
; based upon:
;
; White/Black balance script  for GIMP 1.2
; Copyright (C) 2002 Iccii <iccii@hotmail.com>
; 
; --------------------------------------------------------------------
; 
; This program is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.  
; 
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
; 
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
;

(define (script-grey-point
                        image
                        drawable
                        mode
        )

  (define (floor x) (- x (fmod x 1)))

  (define (curve-point source target i)
        (if (= source 0) target (max (min (floor (* (/ target source) i))  255) 0)))

  (let* ((fg_colour (car (gimp-palette-get-foreground)))
         (bg_colour (car (gimp-palette-get-background)))
         ;
         (source_red    (car   fg_colour))
         (source_green  (cadr  fg_colour))
         (source_blue   (caddr fg_colour))
         (average  (/ (+ source_red source_green source_blue) 3))
         ;
         (target_red   (cond ((= mode 0) average)
                             ((= mode 1) 255)
                             ((= mode 2) (car bg_colour))
                        ))
         (target_green (cond ((= mode 0) average)
                             ((= mode 1) 255)
                             ((= mode 2) (cadr bg_colour))
                        ))
         (target_blue  (cond ((= mode 0) average)
                             ((= mode 1) 255)
                             ((= mode 2) (caddr bg_colour))
                        ))
         ;
         (i 0)
         (num_bytes 256)
         (red-curve   (cons-array num_bytes 'byte))
         (green-curve (cons-array num_bytes 'byte))
         (blue-curve  (cons-array num_bytes 'byte)))

    (gimp-undo-push-group-start image)

    (while (< i num_bytes)
      (aset red-curve   i (curve-point source_red   target_red   i))
      (aset green-curve i (curve-point source_green target_green i))
      (aset blue-curve  i (curve-point source_blue  target_blue  i))
      (set! i (+ i 1)))

    (gimp-curves-explicit drawable RED-LUT   num_bytes red-curve  )
    (gimp-curves-explicit drawable GREEN-LUT num_bytes green-curve)
    (gimp-curves-explicit drawable BLUE-LUT  num_bytes blue-curve )

    (gimp-undo-push-group-end image)
    (gimp-displays-flush)
))

(script-fu-register
  "script-grey-point"
  "<Image>/Script-Fu/Photo/Grey Point..."
  "Linear conversion of RGB colour space.\n\nThe purpose of this script is to transform the foreground colour (as chosen with the colour pick tool) to a chosen target colour. The entire RGB colour space is linearly converted to achieve the desired mapping of the foreground colour. There are three possible choices for the target colour:\n(a) a grey colour of the same density as the foreground colour (desaturation)\n(b) white\n(c) the background colour (as chosen with the colour pick tool)\n\nThe main application area of this script should be the correction of the colour temperature of digital images.\n\nYour feedback is welcome. ;^)"
  "Dr. Martin Rogge <marogge@onlinehome.de>"
  "Dr. Martin Rogge"
  "29/09/2004"
  "RGB*"
  SF-IMAGE    "Image"         0
  SF-DRAWABLE "Drawable"      0
  SF-OPTION   "Foreground transformation"  '("Desaturation" "White" "Background")
)
